"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const from_dir_1 = require("./from-dir");
const string_builder_1 = require("./string-builder");
const fs = require("fs");
const path = require("path");
function build(rootPath) {
    return __awaiter(this, void 0, void 0, function* () {
        let content = new string_builder_1.StringBuilder();
        let home = path.resolve(rootPath);
        let sandboxes = [];
        from_dir_1.fromDir(home, /\.sandbox.ts$/, (filename) => {
            let sandboxPath = filename.replace(home, '.').replace(/.ts$/, '').replace(/\\/g, '/');
            const contents = fs.readFileSync(filename, 'utf8');
            const matchSandboxOf = /\s?sandboxOf\s*\(\s*([^)]+?)\s*\)/g.exec(contents);
            if (matchSandboxOf) {
                const typeName = matchSandboxOf[1].split(',')[0].trim();
                const labelText = /label\s*:\s*['"](.+)['"]/g.exec(matchSandboxOf[0]);
                let scenarioMenuItems = [];
                const scenarioRegex = /\.add\s*\(['"](.+)['"]\s*,\s*{/g;
                let scenarioMatches;
                let scenarioIndex = 1;
                while ((scenarioMatches = scenarioRegex.exec(contents)) !== null) {
                    scenarioMenuItems.push({ key: scenarioIndex, description: scenarioMatches[1] });
                    scenarioIndex++;
                }
                let label = labelText ? labelText[1] : '';
                sandboxes.push({
                    key: sandboxPath,
                    searchKey: `${typeName}${label}`,
                    name: typeName,
                    label: label,
                    scenarioMenuItems
                });
            }
        });
        content.addLine(`export function getSandboxMenuItems() {`);
        content.addLine(`return ${JSON.stringify(sandboxes)};`);
        content.addLine(`}`);
        content.addLine(`export function getSandbox(path: string) {`);
        content.addLine(`switch(path) {`);
        sandboxes.forEach(({ key }) => {
            content.addLine(`case '${key}':`);
            content.addLine(`return import('${key}').then(sandbox => { return sandbox.default.serialize('${key}'); });`);
        });
        content.addLine(`}}`);
        let filePath = path.resolve(home, './sandboxes.ts');
        return new Promise((resolve, reject) => {
            fs.writeFile(filePath, content.dump(), function (err) {
                if (err) {
                    reject(err);
                }
                else {
                    console.log(`Created file: ${filePath}`);
                    resolve(filePath);
                }
            });
        });
    });
}
exports.build = build;
